package org.bitbucket.bradleysmithllc.etlagent.informatica.client;

import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.*;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.*;
import org.bitbucket.bradleysmithllc.etlunit.InformaticaError;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ParameterFile;

import java.io.File;
import java.util.List;
import java.util.Map;

public class AgentClient implements InformaticaRepositoryClient, InformaticaIntegrationServiceClient, InformaticaWebServicesHubClient
{
	private final InformaticaAgentClient informaticaAgentClient;

	private final InformaticaIntegrationService informaticaIntegrationService;
	private final InformaticaRepository informaticaRepository;

	public AgentClient(InformaticaRepository informaticaRepository, String clientId) {
		this.informaticaRepository = informaticaRepository;
		informaticaIntegrationService = null;

		informaticaAgentClient = new InformaticaAgentClient(clientId, informaticaRepository.getInformaticaDomain().getAgentHost(), informaticaRepository.getInformaticaDomain().getAgentPort());
	}

	public AgentClient(InformaticaIntegrationService informaticaIntegrationService, String clientId) {
		this.informaticaIntegrationService = informaticaIntegrationService;
		this.informaticaRepository = informaticaIntegrationService.getInformaticaRepository();

		informaticaAgentClient = new InformaticaAgentClient(clientId, informaticaRepository.getInformaticaDomain().getAgentHost(), informaticaRepository.getInformaticaDomain().getAgentPort());
	}


	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflow(String workFlow, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, null);
	}

	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflow(String workFlow, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, remoteFiles);
	}


	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflowTask(String workFlow, String task, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, null);
	}

	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflowTask(String workFlow, String task, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, remoteFiles);
	}

	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflowTask(String workFlow, String runInstanceName, String task, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, null);
	}

	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflowTask(String workFlow, String runInstanceName, String task, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		ExecuteWorkflowRequestDTO request = new ExecuteWorkflowRequestDTO(folder, workFlow, task, runInstanceName, parmFile);

		// if files are posted, pass them on
		if (remoteFiles != null)
		{
			for (List<RemoteFile> remoteFileList : remoteFiles.values())
			{
				for (RemoteFile remoteFile : remoteFileList)
				{
					request.addRequestFile(remoteFile);
				}
			}
		}

		return informaticaAgentClient.executeWorkflow(request);
	}

	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflow(String workFlow, String runInstanceName, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, runInstanceName, null, folder, parmFile);
	}

	@Override
	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflow(String workFlow, String runInstanceName, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, runInstanceName, null, folder, parmFile, remoteFiles);
	}

	@Override
	public InformaticaIntegrationService getIntegrationService() {
		return informaticaIntegrationService;
	}

	@Override
	public void createConnection(String serverName, String databaseName, String databaseUsername, String databasePassword, String relationalConnectionName) throws Exception, InformaticaError {
		informaticaAgentClient.createConnection(new CreateConnectionRequestDTO(relationalConnectionName, serverName, databaseName, databaseUsername, databasePassword));
	}

	@Override
	public void createFolder(String folder) throws Exception, InformaticaError {
		createFolder(folder, false);
	}

	@Override
	public void createFolder(String folder, boolean temporary) throws Exception, InformaticaError {
		informaticaAgentClient.createFolder(new CreateFolderRequestDTO(folder));
	}

	@Override
	public void deleteFolder(String folder) throws Exception, InformaticaError {
		deleteFolder(folder, true);
	}

	@Override
	public void deleteFolder(String folder, boolean failIfNotExists) throws Exception, InformaticaError {
		informaticaAgentClient.deleteFolder(new DeleteFolderRequestDTO(folder, failIfNotExists));
	}

	@Override
	public void deleteConnection(String connectionName) throws Exception, InformaticaError {
		deleteConnection(connectionName, true);
	}

	@Override
	public void deleteConnection(String connectionName, boolean failIfNotExists) throws Exception, InformaticaError {
		informaticaAgentClient.deleteConnection(new DeleteConnectionRequestDTO(connectionName, failIfNotExists));
	}

	@Override
	public void deleteTemporaryFolders() throws Exception, InformaticaError {
		throw new UnsupportedOperationException();
	}

	@Override
	public void exportWorkflowToXml(String workFlow, String folder, File output) throws Exception, InformaticaError {
		informaticaAgentClient.exportWorkflow(new ExportWorkflowRequestDTO(folder, workFlow, output));
	}

	@Override
	public void exportAllWorkflowsToXml() throws Exception, InformaticaError {
		throw new UnsupportedOperationException();
	}

	@Override
	public void importWorkflowFromXml(String folderPrefix, InformaticaIntegrationService newService, File xmlFile, List<String> flist) throws Exception {
		String name = xmlFile.getName();

		if (name.toLowerCase().endsWith(".xml"))
		{
			name = name.substring(0, name.length() - 4);
		}

		informaticaAgentClient.importWorkflow(new ImportWorkflowRequestDTO(folderPrefix, name, xmlFile, flist));
	}

	@Override
	public void importWorkflowFromXml(String folderPrefix, InformaticaIntegrationService newService, File xml) throws Exception {
		String name = xml.getName();

		if (name.toLowerCase().endsWith(".xml"))
		{
			name = name.substring(0, name.length() - 4);
		}

		informaticaAgentClient.importWorkflow(new ImportWorkflowRequestDTO(folderPrefix, name, xml));
	}

	@Override
	public List<String> listFolders() throws Exception, InformaticaError {
		throw new UnsupportedOperationException();
	}

	@Override
	public int purgeTestFolders() throws Exception {
		return informaticaAgentClient.purgeTestFolders();
	}

	@Override
	public InformaticaRepository getInformaticaRepository() {
		return informaticaRepository;
	}

	@Override
	public InformaticaDomain getInformaticaDomain() {
		return informaticaRepository.getInformaticaDomain();
	}

	@Override
	public void connect() throws Exception {
		// nothing required here
	}

	@Override
	public void dispose() {
		// nothing
	}

	@Override
	public InformaticaWebServicesHub getWebServicesHub() {
		return null;  //To change body of implemented methods use File | Settings | File Templates.
	}
}