package org.bitbucket.bradleysmithllc.etlagent.informatica.client;

import com.google.gson.FieldNamingPolicy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.InstanceCreator;
import com.sun.jersey.api.client.WebResource;
import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.etlagent.client.EtlAgentClient;
import org.bitbucket.bradleysmithllc.etlagent.dto.GenericResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.*;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ParameterFile;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ScopeSection;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ScopeSectionImpl;
import org.bitbucket.bradleysmithllc.webserviceshubclient.regexp.ScopeExpression;

import javax.ws.rs.core.MediaType;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Type;
import java.net.UnknownHostException;
import java.util.List;
import java.util.Map;

public class InformaticaAgentClient extends EtlAgentClient
{
	private final Gson gson = new GsonBuilder()
			.setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_DASHES)
			.registerTypeAdapter(ScopeSection.class, new InstanceCreator<ScopeSection>() {
				@Override
				public ScopeSection createInstance(Type type) {
					return new ScopeSectionImpl(new ScopeExpression("[Global]"));
				}
			})
			.create();

	public InformaticaAgentClient(String clientId) throws UnknownHostException {
		super(clientId);
	}

	public InformaticaAgentClient(String clientId, int port) throws UnknownHostException {
		super(clientId, port);
	}

	public InformaticaAgentClient(String clientId, String hostName) {
		super(clientId, hostName);
	}

	public InformaticaAgentClient(String clientId, String hostName, int port) {
		super(clientId, hostName, port);
	}

	public int purgeTestFolders() throws IOException {
		WebResource webResource = agentRequest("informatica", "purgeTestFolders");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, "{}");

		PurgeTestFoldersResponseDTO dto = gson.fromJson(json, PurgeTestFoldersResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}

		return dto.getNumFoldersRemoved();
	}

	public void createFolder(CreateFolderRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "createInformaticaFolder");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		GenericResponseDTO dto = gson.fromJson(json, GenericResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}
	}

	public void deleteFolder(DeleteFolderRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "deleteInformaticaFolder");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		GenericResponseDTO dto = gson.fromJson(json, GenericResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}
	}

	public void createConnection(CreateConnectionRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "createInformaticaConnection");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		GenericResponseDTO dto = gson.fromJson(json, GenericResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}
	}

	public void deleteConnection(DeleteConnectionRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "deleteInformaticaConnection");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		GenericResponseDTO dto = gson.fromJson(json, GenericResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}
	}

	public void importWorkflow(ImportWorkflowRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "importWorkflowFromXml");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		GenericResponseDTO dto = gson.fromJson(json, GenericResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}
	}

	public void exportWorkflow(ExportWorkflowRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "exportWorkflowToXml");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		ExportWorkflowResponseDTO dto = gson.fromJson(json, ExportWorkflowResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}

		// transfer the response into a file
		FileUtils.write(request.getTarget(), dto.getWorkflowXml());
	}

	public Map<RemoteFile.fileType, List<RemoteFile>> executeWorkflow(ExecuteWorkflowRequestDTO request) throws IOException {
		WebResource webResource = agentRequest("informatica", "executeWorkflow");

		String json = webResource.accept(MediaType.APPLICATION_JSON_TYPE).post(String.class, gson.toJson(request));

		ExecuteWorkflowResponseDTO dto = gson.fromJson(json, ExecuteWorkflowResponseDTO.class);

		if (dto.getResponse() != GenericResponseDTO.response_code.okay)
		{
			throw new IOException(dto.getResponseMessage());
		}

		return dto.getResponseFiles();
	}

	public static void main(String [] argv) throws IOException {
		new InformaticaAgentClient("smoketest", "bsmithvm").purgeTestFolders();
		new InformaticaAgentClient("smoketest", "bsmithvm").deleteFolder(new DeleteFolderRequestDTO("__bsmithvm_test_SHARED_EDW", false));
		new InformaticaAgentClient("smoketest", "bsmithvm").createFolder(new CreateFolderRequestDTO("__bsmithvm_test_SHARED_EDW"));

		File src = new File("/Users/bsmith/git/etl-unit/integration/informatica-integration-test/src/main/informatica/SHARED_EDW/wkf_INT_CONVERSION_TEST.xml");
		File tgt = new File("wkf_INT_CONVERSION_TEST.xml");

		new InformaticaAgentClient("smoketest", "bsmithvm").importWorkflow(new ImportWorkflowRequestDTO("__bsmithvm_test", "wkf_INT_CONVERSION_TEST", src));

		new InformaticaAgentClient("smoketest", "bsmithvm").exportWorkflow(new ExportWorkflowRequestDTO("__bsmithvm_test_SHARED_EDW", "wkf_INT_CONVERSION_TEST", tgt));

		final String [] [] parameters =
				{
						new String[]{"$BadFileName1", "crap"},
						new String[]{"$InputFile1", "inputfile"},
						new String[]{"$OutputFile1", "outputfile"},
						new String[]{"$PMTargetFileDir", "#{targetFiles}"},
						new String[]{"$PMSourceFileDir", "#{sourceFiles}"},
						new String[]{"$PMLookupFileDir", "#{lookupFiles}"},
						new String[]{"$PMBadFileDir", "#{badFiles}"},
						new String[]{"$PMSessionLogDir", "#{sessionLog}"},
						new String[]{"$PMWorkflowLogDir", "#{workflowLog}"},
						new String[]{"$PMSessionLogFile", "s_m_int_conversion_test.log"}
				};

		ParameterFile pf = new ParameterFile();

		ScopeSection ss = pf.appendSection("[Global]");

		for (String[] params : parameters)
		{
			ss.appendParameter(params[0], params[1]);
		}

		ExecuteWorkflowRequestDTO request = new ExecuteWorkflowRequestDTO("__bsmithvm_test_SHARED_EDW", "wkf_INT_CONVERSION_TEST", pf);

		File source1 = new File("source1");
		FileUtils.write(source1, "source1");
		File source2 = new File("source2");
		FileUtils.write(source2, "source2");

		File lookup1 = new File("lookup1");
		FileUtils.write(lookup1, "lookup1");
		File lookup2 = new File("lookup2");
		FileUtils.write(lookup2, "lookup2");

		request.addRequestFile(RemoteFile.newRemoteFile(source1, RemoteFile.fileType.sourceFiles));
		request.addRequestFile(RemoteFile.newRemoteFile(source2, RemoteFile.fileType.sourceFiles));
		request.addRequestFile(RemoteFile.newRemoteFile(lookup1, RemoteFile.fileType.lookupFiles));
		request.addRequestFile(RemoteFile.newRemoteFile(lookup2, RemoteFile.fileType.lookupFiles));

		source2 = new File("inputfile");
		FileUtils.write(source2, "Ordermatic|1D6|x|x|x|x\n" +
				"Ordermatic|0.01|x|x|x|x\n" +
				"Ordermatic|1.045|x|x|x|x\n" +
				"Ordermatic|1.033335|x|x|x|x\n" +
				"Ordermatic|-1D6|x|x|x|x\n" +
				"Ordermatic|-0.01|x|x|x|x\n" +
				"Ordermatic|-1.045|x|x|x|x\n" +
				"Ordermatic|-1.033335|x|x|x|x\n" +
				"SONIC NT|1D6|1D6|1D6|1D6|1D6\n" +
				"SONIC NT|0.01|0.01|0.01|0.01|0.01\n" +
				"SONIC NT|1.045|1.045|1.045|1.045|1.045\n" +
				"SONIC NT|1.033335|x|x|x|x\n" +
				"SONIC NT|-1D6|1D6|1D6|1D6|1D6\n" +
				"SONIC NT|-0.01|0.01|0.01|0.01|0.01\n" +
				"SONIC NT|-1.045|1.045|1.045|1.045|1.045\n" +
				"SONIC NT|-1.033335|x|x|x|x\n" +
				"Ordermatic|1234567890|x123|1.23||1 11\n" +
				"Ordermatic|100000|x|x|x|x\n" +
				"Ordermatic|x|100000|x|x|x\n" +
				"Ordermatic|x|x|100000|x|x\n" +
				"Ordermatic|x|x|x|100000|x\n" +
				"Ordermatic|x|x|x|x|100000\n" +
				"Ordermatic|10000|x|x|x|x\n" +
				"Ordermatic|x|10000|x|x|x\n" +
				"Ordermatic|x|x|10000|x|x\n" +
				"Ordermatic|x|x|x|10000|x\n" +
				"Ordermatic|x|x|x|x|10000\n" +
				"SONIC NT|100000|x|x|x|x\n" +
				"SONIC NT|x|100000|x|x|x\n" +
				"SONIC NT|x|x|100000|x|x\n" +
				"SONIC NT|x|x|x|100000|x\n" +
				"SONIC NT|x|x|x|x|100000\n" +
				"SONIC NT|10000|x|x|x|x\n" +
				"SONIC NT|x|10000|x|x|x\n" +
				"SONIC NT|x|x|10000|x|x\n" +
				"SONIC NT|x|x|x|10000|x\n" +
				"SONIC NT|x|x|x|x|10000\n" +
				"SONIC NT|1}|1}|1}|1}|1}\n" +
				"SONIC NT|1J|1J|1J|1J|1J\n" +
				"SONIC NT|1K|1K|1K|1K|1K\n" +
				"SONIC NT|1L|1L|1L|1L|1L\n" +
				"SONIC NT|1M|1M|1M|1M|1M\n" +
				"SONIC NT|1N|1N|1N|1N|1N\n" +
				"SONIC NT|1O|1O|1O|1O|1O\n" +
				"SONIC NT|1P|1P|1P|1P|1P\n" +
				"SONIC NT|1Q|1Q|1Q|1Q|1Q\n" +
				"SONIC NT|1R|1R|1R|1R|1R\n" +
				"SONIC NT|0|x|x|x|x\n" +
				"SONIC NT|x|0|x|x|x\n" +
				"SONIC NT|x|x|0|x|x\n" +
				"SONIC NT|x|x|x|0|x\n" +
				"SONIC NT|x|x|x|x|0\n" +
				"Ordermatic|0|x|x|x|x\n" +
				"Ordermatic|x|0|x|x|x\n" +
				"Ordermatic|x|x|0|x|x\n" +
				"Ordermatic|x|x|x|0|x\n" +
				"Ordermatic|x|x|x|x|0\n" +
				"SONIC NT|5|05|005|0005|00005\n" +
				"SONIC NT|50|050|0050|00050|000050\n" +
				"SONIC NT|500|0500|00500|000500|0000500\n" +
				"Ordermatic|5|05|005|0005|00005\n" +
				"Ordermatic|50|050|0050|00050|000050\n" +
				"Ordermatic|500|0500|00500|000500|0000500\n");
		request.addRequestFile(RemoteFile.newRemoteFile(source2, RemoteFile.fileType.sourceFiles));

		new InformaticaAgentClient("smoketest", "bsmithvm").executeWorkflow(request);

		String srcText = FileUtils.readFileToString(src);
		String tgtText = FileUtils.readFileToString(tgt);

		new InformaticaAgentClient("smoketest", "bsmithvm").deleteFolder(new DeleteFolderRequestDTO("__bsmithvm_test_SHARED_EDW", true));

		new InformaticaAgentClient("smoketest", "bsmithvm").deleteConnection(new DeleteConnectionRequestDTO("test", false));
		new InformaticaAgentClient("smoketest", "bsmithvm").createConnection(new CreateConnectionRequestDTO("test", "server", "database", "user", "password"));
		new InformaticaAgentClient("smoketest", "bsmithvm").deleteConnection(new DeleteConnectionRequestDTO("test", true));
	}
}