package org.bitbucket.bradleysmithllc.etlagent.informatica.dto;

import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaDomain;
import org.codehaus.plexus.util.StringUtils;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class RemoteFile
{
	public static void copyLocalToRemote(InformaticaDomain informaticaDomain, final Map<fileType, List<RemoteFile>> responseFiles) {
		// now that it is done, scan the output directories for artifacts
		informaticaDomain.getTargetFilesDir().listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				try {
					addResponseFile(RemoteFile.newRemoteFile(file, RemoteFile.fileType.targetFiles), responseFiles);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
				return false;
			}
		});

		informaticaDomain.getBadFilesDir().listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				try {
					addResponseFile(RemoteFile.newRemoteFile(file, RemoteFile.fileType.badFiles), responseFiles);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
				return false;
			}
		});

		informaticaDomain.getSessionLogDir().listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				try {
					addResponseFile(RemoteFile.newRemoteFile(file, RemoteFile.fileType.sessionLog), responseFiles);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
				return false;
			}
		});

		informaticaDomain.getWorkflowLogDir().listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				try {
					addResponseFile(RemoteFile.newRemoteFile(file, RemoteFile.fileType.workflowLog), responseFiles);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
				return false;
			}
		});

		informaticaDomain.getParameterFilesDir().listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				try {
					addResponseFile(RemoteFile.newRemoteFile(file, RemoteFile.fileType.parameterFiles), responseFiles);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
				return false;
			}
		});
	}

	public enum fileType
	{
		lookupFiles("LookupFiles"),
		sourceFiles("SourceFiles"),
		targetFiles("TargetFiles"),
		sessionLog("SessionLog"),
		workflowLog("WorkflowLog"),
		parameterFiles("ParameterFiles"),
		badFiles("BadFiles");

		private final String directoryName;

		private fileType(String directoryName) {
			this.directoryName = directoryName;
		}

		public String getDirectoryName() {
			return directoryName;
		}
	}

	private final fileType type;
	private final String fileName;
	private final String fileContents;

	public RemoteFile(String fileName, String fileContents, fileType type) {
		this.fileName = fileName;
		this.fileContents = fileContents;
		this.type = type;
	}

	public String getFileName() {
		return fileName;
	}

	public String getFileContents() {
		return fileContents;
	}

	public fileType getType() {
		return type;
	}

	public static RemoteFile newRemoteFile(String refName, String contents, fileType type) {
		return new RemoteFile(refName, contents, type);
	}

	public static RemoteFile newRemoteFile(File source, fileType type) throws IOException {
		return newRemoteFile(source, source.getName(), type);
	}

	public static RemoteFile newRemoteFile(File source, String refName, fileType type) throws IOException {
		return new RemoteFile(refName, FileUtils.readFileToString(source), type);
	}

	public boolean sameId(RemoteFile file) {
		return file.fileName.equals(fileName) && file.type == type;
	}

	public static void copyRemoteToLocal(Map<fileType, List<RemoteFile>> sourceFiles1, InformaticaDomain domain) throws IOException {
		// first, recursively purge the workspace
		File workingRoot = domain.getWorkingRoot();
		if (workingRoot.exists()) {
			FileUtils.forceDelete(workingRoot);
		}

		FileUtils.forceMkdir(workingRoot);

		// recreate
		for (fileType type : RemoteFile.fileType.values())
		{
			File targetFiles = domain.getWorkspaceDir(type);
			FileUtils.forceMkdir(targetFiles);
		}

		if (sourceFiles1 != null) {
			for (Map.Entry<RemoteFile.fileType, List<RemoteFile>> sourceEntry : sourceFiles1.entrySet()) {
				List<RemoteFile> value = sourceEntry.getValue();

				for (RemoteFile remoteFile : value) {
					FileUtils.write(new File(domain.getWorkspaceDir(remoteFile.getType()), remoteFile.getFileName()), remoteFile.getFileContents());
				}
			}
		}
	}

	private static void addResponseFile(RemoteFile remoteFile, Map<fileType, List<RemoteFile>> responseFiles) {
		if (!responseFiles.containsKey(remoteFile.getType()))
		{
			responseFiles.put(remoteFile.getType(), new ArrayList<RemoteFile>());
		}

		responseFiles.get(remoteFile.getType()).add(remoteFile);
	}

	@Override
	public String toString() {
		return "RemoteFile{" +
				"type=" + type +
				", fileName='" + fileName + '\'' +
				", fileContents='" + StringUtils.abbreviate(fileContents, 180) + '\'' +
				'}';
	}
}