package org.bitbucket.bradleysmithllc.etlagent.informatica.handler;

import com.fasterxml.jackson.databind.JsonNode;
import org.bitbucket.bradleysmithllc.etlagent.dto.GenericResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.json.informatica.delete_informatica_folder.DeleteInformaticaFolderRequest;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaDomain;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepository;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepositoryClient;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

public class DeleteInformaticaFolder extends AbstractHandler<DeleteInformaticaFolderRequest, GenericResponseDTO>
{
	@Override
	public String getId() {
		return "deleteInformaticaFolder";
	}

	@Override
	public DeleteInformaticaFolderRequest getRequestContainerObject() {
		return new DeleteInformaticaFolderRequest();
	}

	@Override
	public GenericResponseDTO process(JsonNode request, final DeleteInformaticaFolderRequest container) throws Exception {
		System.out.println("Deleting folder");

		String domain = container.getInformaticaDomain();

		String repo = container.getInformaticaRepository();

		// resolve to an informatica domain

		InformaticaDomain informaticaDomain = informaticaConfiguration.getDefaultDomain();

		if (domain != null)
		{
			System.out.println("No domain specified - using the default domain");

			informaticaDomain = informaticaConfiguration.getDomain(domain);

			if (informaticaDomain == null)
			{
				throw new IllegalArgumentException("Domain '" + domain + "' not found");
			}
		}
		else
		{
			System.out.println("No domain specified - using the default domain " + informaticaDomain.getDomainName());
		}

		InformaticaRepository informaticaRepository = null;

		if (repo == null)
		{
			informaticaRepository = informaticaDomain.getDefaultRepository();
		}
		else
		{
			informaticaRepository = informaticaDomain.getRepository(repo);
		}

		if (informaticaRepository == null)
		{
			throw new IllegalArgumentException("Domain '" + domain + "' does not contain repository '" + repo + "'");
		}

		useRepository(informaticaRepository, new RepositoryVisitor() {
			@Override
			public void useClient(InformaticaRepositoryClient client, RuntimeSupport runtimeSupport) throws Exception {
				String folder = container.getFolder();

				boolean fine = true;

				if (container.getFailIfNotExists() != null)
				{
					fine = container.getFailIfNotExists().booleanValue();
				}

				client.deleteFolder(folder, fine);
			}
		});


		return null;
		//return JsonLoader.fromString("{\"response\": \"okay\"}");
	}
}