package org.bitbucket.bradleysmithllc.etlagent.informatica.provider;

import org.bitbucket.bradleysmithllc.etlagent.informatica.client.AgentClient;
import org.bitbucket.bradleysmithllc.etlagent.resources.ContextBase;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

import java.util.ArrayList;
import java.util.List;

/*
			dev_pc_repo: {}
*/
public class InformaticaRepository {
	private final String repositoryName;
	private final InformaticaDomain informaticaDomain;

	private int numRequestsSent = 0;

	private final List<InformaticaIntegrationService> integrationServices = new ArrayList<InformaticaIntegrationService>();
	private final RuntimeSupport runtimeSupport;

	private final List<InformaticaWebServicesHub> informaticaWebServicesHubs = new ArrayList<InformaticaWebServicesHub>();

	private InformaticaRepositoryClient informaticaRepositoryClient;

	public InformaticaRepository(InformaticaDomain domain, String name, RuntimeSupport runtimeSupport) {
		repositoryName = name;
		informaticaDomain = domain;
		this.runtimeSupport = runtimeSupport;

		switch (domain.getClientType()) {
			case AGENT:
				informaticaRepositoryClient = new AgentClient(this, runtimeSupport.getProjectUID());
				break;
			case LOCAL:
				informaticaRepositoryClient = new InformaticaRepositoryClientImpl(this, null, informaticaDomain.getInformaticaBinDir(), runtimeSupport, runtimeSupport.getApplicationLog());
				break;
			case MOCK:
				informaticaRepositoryClient = ClientProxyFactory.getInformaticaRepositoryClientProxy(this, runtimeSupport);
		}
	}

	public List<InformaticaIntegrationService> getIntegrationServices() {
		return integrationServices;
	}

	public List<InformaticaWebServicesHub> getWebServicesHubs() {
		return informaticaWebServicesHubs;
	}

	public InformaticaIntegrationService getDefaultIntegrationService() {
		return integrationServices.size() > 0 ? integrationServices.get(0) : null;
	}

	public InformaticaWebServicesHub getDefaultWebServicesHub() {
		return informaticaWebServicesHubs.size() > 0 ? informaticaWebServicesHubs.get(0) : null;
	}

	public String getRepositoryName() {
		return repositoryName;
	}

	public String getQualifiedName() {
		return informaticaDomain.getDomainName() + "." + repositoryName;
	}

	public InformaticaDomain getInformaticaDomain() {
		return informaticaDomain;
	}

	public void dispose() {
		if (informaticaRepositoryClient != null) {
			informaticaRepositoryClient.dispose();
		}
	}

	public InformaticaIntegrationService getIntegrationService(String isName) {
		for (InformaticaIntegrationService iis : getIntegrationServices()) {
			if (iis.getIntegrationServiceName().equals(isName)) {
				return iis;
			}
		}

		return null;
	}

	public InformaticaWebServicesHub getWebServicesHub(String isName) {
		for (InformaticaWebServicesHub iis : getWebServicesHubs()) {
			if (iis.getName().equals(isName)) {
				return iis;
			}
		}

		return null;
	}

	public InformaticaRepositoryClient getInformaticaRepositoryClient() {
		return informaticaRepositoryClient;
	}

	void request()
	{
		numRequestsSent++;
		informaticaDomain.request();
	}

	public int getNumRequestsSent() {
		return numRequestsSent;
	}
}