package org.bitbucket.bradleysmithllc.etlagent.informatica.provider;

import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.json.DomainsProperty;
import org.bitbucket.bradleysmithllc.etlagent.resources.ContextBase;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

public class InformaticaWebServicesHub
{
	private final String name;
	private final InformaticaDomain informaticaDomain;
	private final InformaticaRepository informaticaRepository;
	private final String hostName;
	private final int port;
	private final RuntimeSupport runtimeSupport;

	private int numRequestsSent = 0;

	private final InformaticaWebServicesHubClient informaticaWebServicesHubClient;

	public InformaticaWebServicesHub(String name, InformaticaRepository informaticaRepository, String hostName, int port, RuntimeSupport runtimeSupport) {
		this.informaticaRepository = informaticaRepository;
		this.hostName = hostName;
		this.port = port;
		this.name = name;

		informaticaDomain = informaticaRepository.getInformaticaDomain();
		this.runtimeSupport = runtimeSupport;

		if (informaticaDomain.getClientType() == DomainsProperty.ClientType.MOCK)
		{
			// return a logging proxy
			informaticaWebServicesHubClient = ClientProxyFactory.getInformaticaWebServicesHubClientProxy(this, runtimeSupport);
		}
		else
		{
			informaticaWebServicesHubClient = null;
		}
	}

	public InformaticaDomain getInformaticaDomain() {
		return informaticaDomain;
	}

	public InformaticaRepository getInformaticaRepository() {
		return informaticaRepository;
	}

	public String getName() {
		return name;
	}

	public String getHostName() {
		return hostName;
	}

	public int getPort() {
		return port;
	}

	public InformaticaWebServicesHubClient getInformaticaWebServicesHubClient(InformaticaIntegrationService target) {
		if (informaticaRepository.getIntegrationService(target.getIntegrationServiceName()) == null)
		{
			throw new IllegalArgumentException("Integration service '" + target.getIntegrationServiceName() + "' not found in repository '" + informaticaRepository.getQualifiedName() + "'");
		}

		return informaticaWebServicesHubClient;
	}

	void request()
	{
		numRequestsSent++;
		informaticaDomain.request();
		informaticaRepository.request();
	}

	public int getNumRequestsSent() {
		return numRequestsSent;
	}
}